//////////////////////////////////////////////////////////////////
//
//  CAL Hit class in ROOT
//
//$Id: S4CALExactHit.cxx,v 1.17 2007/03/20 09:36:10 fujiik Exp $
//
//////////////////////////////////////////////////////////////////

#include <iomanip>
#include <iostream>
#include <cstdlib>

#include "S4CALExactHit.h"
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,15,0)
#include "TMath.h"
#endif
using namespace std;

ClassImp(S4CALExactHit)

// ------------------------------------------------------------------
S4CALExactHit::S4CALExactHit()
              : S4VExactHit(0,                 // hit ID
                            0,                 // track ID
                            0,                 // mother track ID
                            0,                 // PDG code
                            0,                 // charge
                            TVector3(0.,0.,0.),// position
                            0,                 // TOF
                            TVector3(0.,0.,0.),// momentum
                            0,                 // energy
                            0,                 // energy deposit
                            0),                // address ptr 
                fPreHitID(0),
                fCellID(0),
                fIsBarrel(kTRUE),
                fIsEM(kTRUE),
                fCellPosition(0.0, 0.0, 0.0),
                fCellXcm(0.0, 0.0, 0.0),
                fPreHitPtr(0)
{
}

// ------------------------------------------------------------------
S4CALExactHit::S4CALExactHit(Int_t     hitID,       // hit ID
                             Int_t     pcode,       // PDG code
                      const TVector3  &pos,         // position [cm]
                             Double_t  tof,         // TOF      [nsec]
                             Double_t  edep,        // energy deposit [GeV]
                             Int_t     preHitID,    // pre hit ID
                             Int_t     cellID,      // cell ID
                             Int_t     isBarrel,    // energy deposit [GeV]
                             Int_t     isEM,        // energy deposit [GeV]
                       const TVector3 &xcell,       // cell position [cm]
                       const TVector3 &xcm,         // E-weighted pos [cm*GeV]
			     Int_t     trackID,
			     Int_t     mtrackID)
              : S4VExactHit(hitID,             // hit ID
                            trackID,           // track ID
                            mtrackID,          // mother track ID
                            pcode,             // PDG code
                            0,                 // charge
                            pos,               // position
                            tof,               // TOF
                            TVector3(0.,0.,0.),// momentum
                            0,                 // energy
                            edep,              // energy deposit
                            0),                // address ptr 
                fPreHitID(preHitID),
                fCellID(cellID),
                fIsBarrel(isBarrel),
                fIsEM(isEM),
                fCellPosition(xcell),
                fCellXcm(xcm),
                fPreHitPtr(0)
{
}

// ------------------------------------------------------------------
S4CALExactHit::~S4CALExactHit()
{
}

// ------------------------------------------------------------------
void S4CALExactHit::DebugPrint(Option_t *opt) const
{
  cerr << setiosflags(ios::fixed);
  cerr << "S4CALExactHit ====================";
  cerr << " PreHitID=" << fPreHitID;
  cerr << " CellID=" << fCellID;
  cerr << " PDGID=" << GetPDGCode();
  cerr << " IsBarrel=" << fIsBarrel;
  cerr << " IsEM=" << fIsEM;
  cerr << " TOF(nsec)=" << GetTOF();
  cerr << " EnergyDeposit(GeV)=" << GetEnergyDeposit();
  cerr << endl;
  cerr << " CellPos(cm)=(" << fCellPosition.x();
  cerr << "," << fCellPosition.y();
  cerr << "," << fCellPosition.z();
  cerr << ") ";
  cerr << " CellXcm(GeV*cm)=(" << fCellXcm.x();
  cerr << "," << fCellXcm.y();
  cerr << "," << fCellXcm.z();
  cerr << ") ";
  cerr << endl;
}

//---------------------------------------------------------------
Int_t S4CALExactHit::Compare(const TObject *obj) const
{
  switch(GetSortBy()) {
    case kPreHitID: {
         Int_t    phid = static_cast<const S4CALExactHit *>(obj)->fPreHitID;
         return (fPreHitID  == phid ? 0 : (fPreHitID < phid ? -1 : 1));
         break;
    }
    case kCellID: {
         Int_t    cellid = static_cast<const S4CALExactHit *>(obj)->fCellID;
         return (fCellID  == cellid ? 0 : (fCellID < cellid ? -1 : 1));
         break;
    }
    default:
      return S4VExactHit::Compare(obj);
      break;
  }
}
