#ifndef S4CALExactHit_H
#define S4CALExactHit_H

///////////////////////////////////////////////////////////////////////
//
//  JSFJ4 CAL Hit data
//
//$Id: S4CALExactHit.h,v 1.10 2007/01/16 19:15:02 ono Exp $
//
///////////////////////////////////////////////////////////////////////

#include "S4VExactHit.h"
#include <TVector3.h>

class S4CALPreHit;

// *******************************************************
class S4CALExactHit : public S4VExactHit {
 public:
      enum ESortOption { kPreHitID = 21, kCellID };
                         
  S4CALExactHit();
  S4CALExactHit(Int_t     hitID,       // hit ID
                Int_t     pcode,       // PDG code
          const TVector3  &pos,        // position [cm]
                Double_t  tof,         // TOF      [nsec]
                Double_t  edep,        // energy deposit [GeV]
                Int_t     preHitID,    // pre hit ID
                Int_t     cellID,      // cell ID
                Int_t     isBarrel,    // (1,0)=(barrel,endcap)
                Int_t     isEM,        // (1,0)=(EM,HD)
          const TVector3 &xcell,       // cell position [cm]
          const TVector3 &xcm,         // E-weighted pos [cm*GeV]
		Int_t     trackID,     // track ID
		Int_t     mtrackID);   // mother track ID
  virtual ~S4CALExactHit();

  // virtual functions in S4VExactHit
  virtual Int_t Compare(const TObject *) const;

  inline Int_t        GetPreHitID    () const { return fPreHitID;     }
  inline Int_t        GetCellID      () const { return fCellID;       }
  inline Bool_t       IsBarrel       () const { return fIsBarrel;     }
  inline Bool_t       IsEM           () const { return fIsEM;         }
  inline TVector3     GetCellPosition() const { return fCellPosition; }
  inline TVector3     GetCellXcm     () const { return fCellXcm;      }
  inline S4CALPreHit* GetPreHitPtr   ()       { return fPreHitPtr;    }

  inline void         SetPreHitPtr(S4CALPreHit *p){fPreHitPtr = p; }
 
  virtual void DebugPrint(Option_t *opt = "") const; //overrides TObject's

 private:
  Int_t        fPreHitID;         // PreHitID
  Int_t        fCellID;           // CellID
  Bool_t       fIsBarrel;         // IsBarrel
  Bool_t       fIsEM;             // IsEM
  TVector3     fCellPosition;     // Cell position in cm
  TVector3     fCellXcm;          // Edep-weighted position
  S4CALPreHit *fPreHitPtr;        // pointer to pre hit 
  
  ClassDef(S4CALExactHit, 2)  // S4CALExactHit class
};

#endif
