//*************************************************************************
//* ----------------
//* S4CALMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*
//* (Provides)
//*	S4CALMCTruthBuf		: raw data buffer
//*	S4CALMCTruth		: steering class
//* (Update Record)
//*	2004/11/24  S.Yamamoto	Original version.
//*************************************************************************

#include "S4CALMCTruth.h"
#include "S4CALExactHit.h"
#include "S4CALMCTruthBuf.h"
#include <iostream>
using namespace std;

ClassImp(S4CALMCTruth)

//=====================================================================
//* constructor -------------------------------------------------------

S4CALMCTruth::S4CALMCTruth(const Char_t *fname, 
                           const Char_t *name,
                           const Char_t *title)
            : S4VMCTruth(name, title),
              fInputFileName((Char_t *)fname)
{
   fEventBuf = new S4CALMCTruthBuf(this);
}

//=====================================================================
//* destructor --------------------------------------------------------

S4CALMCTruth::~S4CALMCTruth()
{
   if(fEventBuf) delete fEventBuf;
   if(!fInputStream) fInputStream->close();
}

//=====================================================================
//* BeginRun  --------------------------------------------------------

Bool_t S4CALMCTruth::BeginRun(Int_t nrun)
{
    return S4VMCTruth::BeginRun(nrun);
}

//=====================================================================
//* Process   --------------------------------------------------------

Bool_t S4CALMCTruth::Process(Int_t ev)
{
   S4VMCTruth::Process(ev);
   S4CALMCTruthBuf *bufp = (S4CALMCTruthBuf *)fEventBuf;

   if (fFile->IsWritable()) { // JSFJ4
      if (bufp->GetInputBufPtr()) {
         TObjArray *ehitbufp   = bufp->GetExactHitsPtr(); 
         ehitbufp->Clear();
         TIter next(bufp->GetInputBufPtr());
         TObject *objp = 0;
         while ((objp = next())) ehitbufp->Add(objp);
      } else { // Read ASCII data
         bufp->ReadFile();
#if 0
	 S4CALExactHit::SetSortBy(S4CALExactHit::kPreHitID);
	 bufp->GetExactHitsPtr()->UnSort();
	 bufp->GetExactHitsPtr()->Sort();
#endif
      } 
      bufp->UpdatePtrs();
#if 0
      DebugPrint();
#endif
      return kTRUE;
   }
   else {
      cerr << "S4CALMCTruth::Process ExacthitsBuffer is not updated. " << endl;
      return kTRUE;
   }
}


//=====================================================================
//* DebugPrint -------------------------------------------------------

void S4CALMCTruth::DebugPrint(const Char_t *)
{
   S4CALMCTruthBuf *bufp = (S4CALMCTruthBuf *)fEventBuf;
   TIter next(bufp->GetExactHitsPtr()); 
   S4CALExactHit *ehitp;
   while ((ehitp = static_cast<S4CALExactHit *>(next()))) {
      ehitp->DebugPrint();
   }
}
