//*************************************************************************
//* -------------------
//* S4HISTMCTruth Class
//* -------------------
//* (Description)
//*     A class to book keep history information.
//* (Requires)
//* (Provides)
//*	S4HISTMCTruthBuf	: data buffer
//*	S4HISTMCTruth		: steering class
//* (Update Record)
//*	2006/02/03  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include "S4HISTMCTruth.h"
#include "S4HISTMCTruthBuf.h"
#include "S4BreakPoint.h"

using namespace std;

ClassImp(S4HISTMCTruth)

//=====================================================================
//* constructor -------------------------------------------------------

S4HISTMCTruth::S4HISTMCTruth(const Char_t *fname, 
                             const Char_t *name,
                             const Char_t *title)
           : S4VMCTruth(name, title),
             fInputFileName((Char_t *)fname)

{
}

//=====================================================================
//* destructor --------------------------------------------------------

S4HISTMCTruth::~S4HISTMCTruth()
{
   if(fEventBuf) delete fEventBuf;
   if(!fInputStream) fInputStream->close();
}

//=====================================================================
//* Initialize  -------------------------------------------------------

Bool_t S4HISTMCTruth::Initialize()
{
   if (!fEventBuf) fEventBuf = new S4HISTMCTruthBuf(this);

   return S4VMCTruth::Initialize();
}

//=====================================================================
//* BeginRun  ---------------------------------------------------------
                                                                                
Bool_t S4HISTMCTruth::BeginRun(Int_t nrun)
{
    return S4VMCTruth::BeginRun(nrun);
}

//=====================================================================
//* Process   --------------------------------------------------------

Bool_t S4HISTMCTruth::Process(Int_t ev)
{
   S4VMCTruth::Process(ev);
   S4HISTMCTruthBuf *bufp = static_cast<S4HISTMCTruthBuf *>(fEventBuf);
   if ( fFile->IsWritable() ) {
      if (!bufp->GetInputBufPtr()) {
         bufp->GetExactHitsPtr()->Delete();
         bufp->ReadFile();
      }
   } else {
      cerr << "S4HISTMCTruth::Process ExacthitsBuffer is not updated. " << endl;
   }
   return kTRUE;
}


//=====================================================================
//* DebugPrint -------------------------------------------------------

void S4HISTMCTruth::DebugPrint(const Char_t *opt)
{
   cerr << " ---- S4HISTMCTruth::DebugPrint ------------- " << endl;
   S4BreakPoint::PrintAll();
}
