//*************************************************************************
//* -------------------
//* S4HISTMCTruth Class
//* -------------------
//* (Description)
//*   	A class to store history information.
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4HISTMCTruthBuf		: MCTruth data buffer
//* (Update Record)
//*	2006/02/03  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include "S4HISTMCTruthBuf.h"
#include "S4VMCTruthBuf.h"
#include "S4BreakPoint.h"

using namespace std;

ClassImp(S4HISTMCTruthBuf)


//===========================================================
// Constructor -----------------------------------------------

S4HISTMCTruthBuf::S4HISTMCTruthBuf(const Char_t  *name,
                                   const Char_t  *title, 
                                   S4HISTMCTruth *module,
                                   Int_t          size) 
              : S4VMCTruthBuf(name, title, module, size),
                fInputBufPtr(0),
                fDone(kFALSE)
{
}

S4HISTMCTruthBuf::S4HISTMCTruthBuf(S4HISTMCTruth *module,
                                 const Char_t *name, 
                                 const Char_t *title,
                                 Int_t         size) 
              : S4VMCTruthBuf(name, title, module, size),
                fInputBufPtr(0),
                fDone(kFALSE)
{
   cerr << "constructor for S4HISTMCTruthBuf is called..." << endl;
}

//===========================================================
// Destructor -----------------------------------------------

S4HISTMCTruthBuf::~S4HISTMCTruthBuf()
{
}

//===========================================================
// ReadFile -----------------------------------------------

Bool_t S4HISTMCTruthBuf::ReadFile()
{
#if 0
   cerr << " S4HISTMCTruthBuf::ReadFile() is called..." << endl;
#endif
   ifstream & in = *S4VMCTruth::GetInputStreamPtr();
   // Seek HIST header -------//

   Int_t eventno = Seek("*****_BreakPoint_output_start_*****");
   if (eventno < 0)  return kTRUE;  // -ve means faiure to find one


   // read data -------//
   static Int_t kMaxBuf = 1024;
   Char_t  dummy[kMaxBuf];
   Int_t    pcode;
   Double_t posx, posy, posz; 
   Double_t tof;
   TVector3 pos;
   Int_t    trackID, motherTrackID;
   Double_t etot, mx, my, mz, charge;
   TVector3 mom;

   Int_t    hitID = 0;
  
   while (!in.eof()) {
     in.getline(dummy,kMaxBuf);
     if (! strncmp(dummy,"*****_BreakPoint_output_end_*****" ,33) ) break;
     istringstream ins(dummy);
     ins >> trackID   >> motherTrackID  
         >> etot 
         >> mx        >> my      >> mz  
         >> pcode     >> charge  >> tof
         >> posx      >> posy    >> posz ;

     pos.SetXYZ(posx, posy, posz);
     mom.SetXYZ(mx, my, mz);
   
     S4BreakPoint *datum 
	= new S4BreakPoint(hitID,                // hit ID
			  trackID,               // track ID 
			  motherTrackID,         // mother track ID
			  pcode,                 // PDG code
			  charge,                // charge
			  pos,                   // Injection position [cm]
			  tof,                   // TOF      [nsec]
			  mom,                   // momentum [GeV] 
			  etot);                 // energy
      GetExactHitsPtr()->Add(datum);	
#if 0
      datum->DebugPrint();
#endif
      hitID++;
   }
   SetInputBufPtr(GetExactHitsPtr());
   SetInputBufPtr(0);

   return kTRUE;
}
 
//===========================================================
// Update  --------------------------------------------------

void S4HISTMCTruthBuf::Update()
{
   if (fDone || !GetInputBufPtr()) return;
   fDone = kTRUE;

   S4BreakPoint::Clear();

   TIter next(GetInputBufPtr());
   S4BreakPoint *bpp;
   while ((bpp = static_cast<S4BreakPoint *>(next()))) {
     S4BreakPoint::fgBreakPointMap.insert(
       pair<Int_t, S4BreakPoint*>(bpp->GetTrackID(), bpp));
     bpp->TObjArray::Clear();
   } 

   next.Reset();
   while ((bpp = static_cast<S4BreakPoint *>(next()))) {
     Int_t         j4ptkid = bpp->GetParentID();
     S4BreakPoint *pbpp    = S4BreakPoint::GetBreakPointPtr(j4ptkid);
     if (pbpp) pbpp->Add(bpp);
     else      bpp->SetBit(BIT(20));
   } 
}
