#ifndef S4VMCTRUTHBUF_H
#define S4VMCTRUTHBUF_H
//*****************************************************************************
//* -------------------
//* S4VMCTruthBuf Class
//* -------------------
//*
//* (Description)
//*	This set of classes reads ascii output of JUPITER and
//*	put them into root objects.
//*
//* (Requires)
//*	S4VMCTruth
//*	TObjArray
//*
//* (Provides)
//*	S4VMCTruthBuf: Monte Carlo Truth data buffer
//*
//* (Update Record)
//*	2001/08/21  K.Hoshina	Original version.
//*
//*****************************************************************************

#include <fstream>
#include <sstream>
#include "JSFModule.h"

#include "S4VMCTruth.h"
#include "TObjArray.h"

//-----------------------------------------------------------------------------
// -------------------
// Class S4VMCTruthBuf
// -------------------

class S4VMCTruthBuf : public JSFEventBuf {

  friend class S4VMCTruth;

  //---------------------------------------------------------------------------
  // =======
  // Methods
  // =======
  //
  // --------------
  // Public Methods
  // --------------
  // 
  public:
     //------------------------------------------------------------------------
     // Ctors and dtor
     //---------------
     S4VMCTruthBuf(const Char_t *name    = "MyMCTruthBuf", 
                   const Char_t *title   = "My MC Truth Buffer", 
                   S4VMCTruth *module  = 0, 
                   Int_t       bufsize = 20000);
     S4VMCTruthBuf(S4VMCTruth *module, 
                   const Char_t *name, 
                   const Char_t *title, 
                   Int_t       bufsize = 20000);

     virtual ~S4VMCTruthBuf();

     //------------------------------------------------------------------------
     // Getters and setters
     //--------------------

     inline virtual TObjArray  * GetExactHitsPtr () { return &fExactHits; }
    
     //------------------------------------------------------------------------
     // Utility functions
     //------------------
     virtual Bool_t ReadFile() { return kTRUE; }
     virtual Int_t  Seek(const Char_t *tag = "*******_End_of_event_");

  //---------------------------------------------------------------------------
  // ============
  // Data Members
  // ============
  //
  private:
     TObjArray	       fExactHits;     //  Array of hits

     static Int_t      fgEventNo;      //! event No

     ClassDef(S4VMCTruthBuf, 1)	// Baby raw data buffer
};

#endif
