#ifndef S4CALCLUSTER_H
#define S4CALCLUSTER_H

#include "TObjArray.h"
#include "TVector3.h"

#include "TAttLockable.h"
#include "TAttDrawable.h"

class S4CALHit;
class S4CALHitFilter;

class S4CALCluster : public TObjArray, 
                     public TAttLockable, 
                     public TAttDrawable {
public:
  S4CALCluster(S4CALHit *chp = 0);
  virtual ~S4CALCluster();

  Double_t   GetEnergy  () { if (!fDone) Update(); return fEnergy;   }
  TVector3 & GetMomentum() { if (!fDone) Update(); return fMomentum; }
  Double_t   GetTime    () { if (!fDone) Update(); return fTime;     }
  TVector3 & GetPosition() { if (!fDone) Update(); return fPosition; }
  Double_t   GetFracEM  () { if (!fDone) Update(); return fFracEM;   }

  using TAttDrawable::Draw;
  virtual void Draw(Int_t color, const Char_t *opt = "");

  // Service method
  virtual Bool_t    Append(S4CALHit *chp);    // Append hit

  static  S4CALHitFilter &GetFilter();
  static  void            SetFilter(S4CALHitFilter *fp = 0);

  static  Double_t  GetEMCut           ()                { return fgEMCut;              } 
  static  Double_t  GetCalibFactorForEM()                { return fgCalibFactorForEM;   }
  static  Double_t  GetCalibFactorForHD()                { return fgCalibFactorForHD;   }
  static  void      SetEMCut           (Double_t value)  { fgEMCut             = value; }
  static  void      SetCalibFactorForEM(Double_t value)  { fgCalibFactorForEM  = value; }
  static  void      SetCalibFactorForHD(Double_t value)  { fgCalibFactorForHD  = value; }

  void DebugPrint(const Char_t *opt = "");

private: 
  using TObjArray::Add;
  virtual void      Update();    // Update cluster information

private:
  Double_t       fEnergy;        // cluster energy   [GeV]
  TVector3       fMomentum;      // cluster momentum [GeV]
  Double_t       fTime;          // cluster time     [ns]
  TVector3       fPosition;      // cluster position [cm]
  Double_t       fFracEM;        // energy fraction in EM
  Bool_t         fDone;          // (1,0) = (clustering done, not yet)

  static S4CALHitFilter *fgFilterPtr;        //! hit filter object
  static Double_t        fgEMCut;            //! cut for CalibrationFactor
  static Double_t        fgCalibFactorForEM; //! CalibrationFactor for EM
  static Double_t        fgCalibFactorForHD; //! CalibrationFactor for HD

  ClassDef(S4CALCluster, 1) // CAL cluster class
};
#endif
