//*************************************************************************
//* --------------------
//* S4CALClusterMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes CALClusters.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2004/09/04  S.Yamamoto	Original version.
//*************************************************************************

#include <iostream>
#include "S4CALCluster.h"
#include "S4CALClusterMakerBuf.h"
#include "S4CALHitMaker.h"
#include "S4CALHitMakerBuf.h"
#include "S4CALHit.h"
#include "S4CALExactHit.h"
#include "S4CALPreHit.h"

ClassImp(S4CALClusterMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
S4CALClusterMakerBuf::S4CALClusterMakerBuf(S4CALClusterMaker *module, 
                                           const Char_t      *name, 
                                           const Char_t      *title, 
                                           Int_t bufsize)
		    : JSFEventBuf(name, title, (JSFModule*)module),
                      fOutputBufPtr(0),
                      fBufSize(bufsize)
{
}

//=====================================================================
//* destructor --------------------------------------------------------
S4CALClusterMakerBuf::~S4CALClusterMakerBuf()
{
  if (fOutputBufPtr) fOutputBufPtr->Delete();
  delete fOutputBufPtr;
}

//=====================================================================
//* GetOutputBufPtr ---------------------------------------------------
TObjArray* S4CALClusterMakerBuf::GetOutputBufPtr()
{
  if (!fOutputBufPtr) fOutputBufPtr = new TObjArray(fBufSize);
  return fOutputBufPtr;
}

//=====================================================================
//* DebugPrint --------------------------------------------------------
 
void S4CALClusterMakerBuf::DebugPrint(const Char_t *)
{
  if (fOutputBufPtr && fOutputBufPtr->GetEntries()) {
    TIter next(fOutputBufPtr);
    S4CALCluster *cp = 0;
    while ((cp = static_cast<S4CALCluster *>(next()))) {
       cp->DebugPrint();
    }
  }
}

//=====================================================================
//* MakeClusters ------------------------------------------------------
void S4CALClusterMakerBuf::MakeClusters(TObjArray *hits)
{
  if (fOutputBufPtr) fOutputBufPtr->Delete();
  GetOutputBufPtr()->SetOwner(); // buffer owns clusters 
  //--
  // First check if there are any hits to cluster
  //--
  TObjArray        &calHits = *hits;
  if (!&calHits || calHits.GetEntries() == 0) return; 
  //--
  // Sort hits by parent ID
  //--
  S4CALHit::SetSortBy(S4CALHit::kPreHitID);  // sort hits by prehitID
  calHits.UnSort();
  calHits.Sort();
  //--
  // Check if any corresponding hybrid track there
  //--
  TIter nexthit(&calHits); // prepare hit iterator
  S4CALHit      *hitp  = static_cast<S4CALHit *>(nexthit());
  //--
  // First process CAL hits with parent hits
  //--
  // Create 1st cluster
  S4CALCluster *curcp  = new S4CALCluster(hitp); // new 1st cluster

  // Then create the rest
  while ((hitp = static_cast<S4CALHit *>(nexthit()))) { // loop over remaining hits
    if (!curcp->Append(hitp)) {  // try append hit to current cluster
      fOutputBufPtr->Add(curcp); // failed, end of current cluster, add current to buffer 
      curcp = new S4CALCluster(hitp); // start new cluster
    }
  }
  fOutputBufPtr->Add(curcp); // add last cluster to buffer
#ifdef __DEBUG__
  DebugPrint();
#endif
  //--
  // That's it, folks!
  //--
}
