#include "S4CALHit.h"
#include "S4CALExactHit.h"
#include <iostream>
#include <limits>

ClassImp(S4CALHit)

S4CALHit::ESortOption S4CALHit::fgSortBy = kPreHitID;

S4CALHit::S4CALHit(S4CALExactHit *ehp)
        : fCALType      (kUnknown),
          fPreHitID     (ehp ? ehp->GetPreHitID() : -999999),
          fCellID       (ehp ? ehp->GetCellID  () : -999999),
          fEnergyDeposit(0.),
          fTime         (0.),
          fDone         (kFALSE)
{
  if (ehp) {
    Add(ehp);
    if (ehp->IsEM()) {
      fCALType |= ehp->IsBarrel() ? kBaEm : kEcEm;
    } else {
      fCALType |= ehp->IsBarrel() ? kBaHd : kEcHd;
    }
  }
}

S4CALHit::~S4CALHit()
{
}

void S4CALHit::Merge(S4CALExactHit *ehp)
{
  if (ehp) {
    Add(ehp);
    if (ehp->IsEM()) {
      fCALType |= ehp->IsBarrel() ? kBaEm : kEcEm;
    } else {
      fCALType |= ehp->IsBarrel() ? kBaHd : kEcHd;
    }
    fDone = kFALSE;
  }
}

void S4CALHit::Update()
{
  if (fDone) return;
  fDone = kTRUE;

  fEnergyDeposit = 0.;
  fPosition.SetXYZ(0.,0.,0.);
  fTime   = std::numeric_limits<int>::max();

  TIter next(this);
  S4CALExactHit *ehp;
  while ((ehp = static_cast<S4CALExactHit *>(next()))) {
    fEnergyDeposit += ehp->GetEnergyDeposit();
    fPosition      += ehp->GetEnergyDeposit() * ehp->GetPosition();
    if (ehp->GetTOF()<fTime) fTime = ehp->GetTOF();
  }
  if (fEnergyDeposit > 0.) fPosition *= 1./fEnergyDeposit;
  else                     fPosition.SetXYZ(0.,0.,0.);
}

void S4CALHit::Draw(Int_t color, const Char_t *opt)
{
}

void S4CALHit::DebugPrint(const Option_t *opt)
{
  S4CALExactHit *ehp = static_cast<S4CALExactHit *>(At(0));
  using namespace std;
   cerr << " -- S4CALHit::DebugPrint ------------------------ " << endl
        << "  htPtr = " << this                                 << endl
        << "  preID = " << GetPreHitID()                        << endl
        << "  Thit  = " << GetTime    ()                        << endl
        << "  Xhit  = " << "(" << GetPosition().X() << ", "
                               << GetPosition().Y() << ", "
                               << GetPosition().Z() << ")"      << endl
        << "  Edep  = " << GetEnergyDeposit()                   << endl
        << "  pdgc  = " << ehp->GetPDGCode()                    << endl
        << " ------------------------------------------------ " << endl;
}

Bool_t S4CALHit::IsSortable() const
{
  return kTRUE;
}

Int_t S4CALHit::Compare(const TObject *obj) const
{
  switch(GetSortBy()) {
    case kPreHitID: {
         Int_t    phid = static_cast<const S4CALHit *>(obj)->fPreHitID;
         return (fPreHitID  == phid ? 0 : (fPreHitID < phid ? -1 : 1));
         break;
    }
    case kCellID: {
         Int_t    cellid = static_cast<const S4CALHit *>(obj)->fCellID;
         return (fCellID  == cellid ? 0 : (fCellID < cellid ? -1 : 1));
         break;
    }
    default:
      return 0;
      break;
  }
}
