#ifndef S4CALHIT_H
#define S4CALHIT_H

#include "TObjArray.h"
#include "TVector3.h"
#include "TAttLockable.h"
#include "TAttDrawable.h"

class S4CALExactHit;

class S4CALHit : public TObjArray, public TAttLockable, public TAttDrawable {
public:
      enum ECALType    { kUnknown  = 0x0000,   // Calorimeter Type not decided
                         kBaEm     = 0x0001,   // Barrel EM
                         kBaHd     = 0x0002,   // Barrel HD
                         kEcEm     = 0x0004,   // Endcap EM
                         kEcHd     = 0x0008 }; // Endcap HD       
      enum ESortOption { kPreHitID = 31, kCellID };

  S4CALHit(S4CALExactHit *ehp = 0);
  virtual ~S4CALHit();

  virtual Int_t    GetCALType      () const  { return fCALType;  }
  virtual Int_t    GetPreHitID     () const  { return fPreHitID; }
  virtual Double_t GetEnergyDeposit() { if (!fDone) Update(); return fEnergyDeposit; }
  virtual Double_t GetTime         () { if (!fDone) Update(); return fTime;          }
  virtual TVector3 GetPosition     () { if (!fDone) Update(); return fPosition;      }

  virtual void     Merge(S4CALExactHit *ehp);

  virtual Bool_t   IsSortable() const;
  virtual Int_t    Compare(const TObject *objp) const;

  static  Int_t    GetSortBy ()                      { return (Int_t)fgSortBy;      }
  static  void     SetSortBy (Int_t opt = kPreHitID) { fgSortBy = (ESortOption)opt; }

  using TAttDrawable::Draw;  
  void  Draw(Int_t color, const Char_t *opt = "");

  void  DebugPrint(const Option_t *opt = "");

private: 
  using TObjArray::Add;
  virtual void      Update();                      // Update hit information

private:
  Int_t    fCALType;          // Calorimeter type
  Int_t    fPreHitID;         // prehit ID
  Int_t    fCellID;           // cell ID
  Double_t fEnergyDeposit;    // hit energy   [GeV]
  Double_t fTime;             // hit time     [ns]
  TVector3 fPosition;         // hit position [cm]
  Bool_t   fDone;             // (1,0) = (info updated, not yet)

  static ESortOption  fgSortBy; //! sorting option

  ClassDef(S4CALHit, 1) // CAL hit class
};
#endif
