//*************************************************************************
//* --------------------
//* S4CPFOMakerBuf
//* --------------------
//* (Description)
//*    This set of classes makes CALClusters.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2004/09/04  S.Yamamoto	Original version.
//*************************************************************************

#include "S4CPFOMakerBuf.h"
#include "S4PFObject.h"
#include "S4CALCluster.h"
#include "S4CALHit.h"
#include "S4HYBTrackMakerBuf.h"
#include "S4CALExactHit.h"
#include "S4CALPreHit.h"
#include <iostream>

ClassImp(S4CPFOMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
S4CPFOMakerBuf::S4CPFOMakerBuf(S4CPFOMaker  *module, 
                               const Char_t *name, 
                               const Char_t *title)
		    : JSFEventBuf(name,title,module),
                      fOutputBufPtr(0)
{
}

//=====================================================================
//* destructor --------------------------------------------------------
S4CPFOMakerBuf::~S4CPFOMakerBuf()
{
  if (fOutputBufPtr) fOutputBufPtr->Delete();
  delete fOutputBufPtr;
}

//=====================================================================
//* GetOutputBufPtr ---------------------------------------------------
TObjArray* S4CPFOMakerBuf::GetOutputBufPtr()
{
  if (!fOutputBufPtr) fOutputBufPtr = new TObjArray();
  return fOutputBufPtr;
}

//=====================================================================
//* DebugPrint --------------------------------------------------------
 
void S4CPFOMakerBuf::DebugPrint(const Char_t *opt)
{
  using namespace std;
  if (fOutputBufPtr && fOutputBufPtr->GetEntries()) {
    cerr << "==== S4CPFOMakerBuf::DebugPrint ================" << endl
         << " No of PFOs = " << fOutputBufPtr->GetEntries()    << endl;
    TIter next(fOutputBufPtr);
    S4PFObject *pfop = 0;
    while ((pfop = static_cast<S4PFObject *>(next()))) {
       pfop->DebugPrint(opt);
    }
  }
}

//=====================================================================
//* MakePFOs ----------------------------------------------------------
void S4CPFOMakerBuf::MakePFOs(TObjArray *cap)
{
  if (fOutputBufPtr) fOutputBufPtr->Delete();
  else               GetOutputBufPtr();
  if (!cap || !cap->GetEntries()) return; 

  TIter next(cap);
  S4CALCluster *cp;
  while ((cp = static_cast<S4CALCluster *>(next()))) {
    S4CALHit      *chp     = static_cast<S4CALHit *>(cp->At(0));
    S4CALExactHit *ehp     = static_cast<S4CALExactHit *>(chp->At(0));
    Int_t          j4trkid = ehp->GetPreHitPtr()->GetTrackID();
    S4HYBTrack    *tp      = S4HYBTrackMakerBuf::GetHYBTrackPtr(j4trkid);
    fOutputBufPtr->Add(new S4PFObject(cp, tp));
  }
#ifdef __DEBUG__
  DebugPrint();
#endif
}
