//
// cpfa.C
//
//

class S4CPFOMaker;
void UserAnalysis(S4CPFOMaker *cpmp);
TFile    *foutp;

//---------------------------------------------------------------------------------------
//  Cheated PFA Steering Script
//---------------------------------------------------------------------------------------
void cpfa(Int_t run = 0)
{
  gSystem->Load("../lib/libJSF.so");
  gSystem->Load("../lib/libS4Utils.so");
  gSystem->Load("../lib/libS4MCTruth.so");
  gSystem->Load("../lib/libS4HISTMCTruth.so");
  gSystem->Load("../lib/libS4CALMCTruth.so");
  gSystem->Load("../lib/libS4HYBTrackMaker.so");
  gSystem->Load("../lib/libS4CALHitMaker.so");
  gSystem->Load("../lib/libS4CALClusterMaker.so");
  gSystem->Load("../lib/libS4CPFOMaker.so");

  gErrorAbortLevel = kWarning + 1; 
       
  JSFSteer *jsf = new JSFSteer();
    
  foutp = new TFile("jsf.root","RECREATE"); //  Output file.
  TFile & fout = *foutp;
  
  if (!run) run = jsf->Env()->GetValue("JSFSteer.RunNo",1);
  cerr << " RunNo = " << run << endl;

  jsf->SetOutput(fout);

  Int_t nevent=jsf->Env()->GetValue("JSFSteer.Nevent",10);  
  Int_t minevt=0;
  Int_t maxevt=minevt+nevent;

  S4VMCTruth::SetInputFileName("../../../hit.dat");

  S4HISTMCTruth     *histmctruth   = new S4HISTMCTruth();  
  S4CALMCTruth      *calmctruth    = new S4CALMCTruth();  
  S4HYBTrackMaker   *hybtmaker     = new S4HYBTrackMaker();  
  S4CALHitMaker     *calhitmaker   = new S4CALHitMaker();  
  S4CALClusterMaker *calclustmaker = new S4CALClusterMaker();  
  S4CPFOMaker       *cpfomaker     = new S4CPFOMaker();  

  histmctruth  ->SetMakeBranch(kFALSE);
  calmctruth   ->SetMakeBranch(kFALSE);
  hybtmaker    ->SetMakeBranch(kFALSE);
  calhitmaker  ->SetMakeBranch(kFALSE);
  calclustmaker->SetMakeBranch(kFALSE);
  cpfomaker    ->SetMakeBranch(kFALSE);

  jsf->Initialize();
    
  jsf->BeginRun(run);      // Set run number

  for (Int_t ev=minevt; ev<=maxevt; ev++) {
    if (!jsf->Process(ev)) {
      cerr << "jsf->Process is faild." << endl;
      jsf->FillTree();
      jsf->Clear();
      break;
    } else {
      UserAnalysis(cpfomaker);
    }
    jsf->FillTree();
    jsf->Clear();
  }
  jsf->Terminate();
  fout.Write();
}

class S4CPFOMakerBuf;
void DrawPFOs(S4CPFOMakerBuf *cpmbp, const Option_t *opt = "");

static TH1D *hEtot = 0;
static TH1D *hMass = 0;

//---------------------------------------------------------------------------------------
//  User Optional Analysis
//---------------------------------------------------------------------------------------
void UserAnalysis(S4CPFOMaker *cpmp)
{
  cerr << "UserAnalysis called" << endl;
  S4CPFOMakerBuf *cpmbp = static_cast<S4CPFOMakerBuf *>(cpmp->EventBuf());
  //--
  // Print out PFOs
  //--
#if 1
  cpmbp->DebugPrint();
  S4BreakPoint::PrintAll("primaryonly");
  //S4BreakPoint::PrintAll("withpfo");
  TLorentzVector qcm;
  TIter next(cpmbp->GetOutputBufPtr());
  S4PFObject *pfop;
  while ((pfop = static_cast<S4PFObject *>(next()))) {
    qcm += *pfop;
  }
  cerr << " Mass = " << qcm.M() << endl;
  if (!hMass) hMass = new TH1D("hMass","",20000,0.,200.);
  hMass->Fill(qcm.M());

  cerr << " Evis = " << qcm.E() << endl;
  if (!hEtot) hEtot = new TH1D("hEtot","",20000,0.,200.);
  hEtot->Fill(qcm.E());
#else
  cpmbp->DebugPrint("detail");
  S4BreakPoint::PrintAll();
#endif
  //--
  // Display PFOs
  //--
  if (!gROOT->IsBatch()) DrawPFOs(cpmbp);
}

static TCanvas     *cvp    = 0;
static TTUBE       *itubep = 0;
static TTUBE       *otubep = 0;
static TRotMatrix  *orotp  = 0;
static TNode       *onodep = 0;
static TNode       *inodep = 0;

//---------------------------------------------------------------------------------------
//  Very Primitive Event Dislpay
//---------------------------------------------------------------------------------------
void DrawPFOs(S4CPFOMakerBuf *cpmbp, const Option_t *opt)
{
   //--
   // Initialize Event Display
   //--
   if (!cvp) {
      cvp = new TCanvas("OED", "Event Display", 400, 10, 610, 610);
   } else {
      cvp->cd();
      cvp->Clear();
   }
   TView *vwp = TView::CreateView(1,0,0);
   vwp->SetRange(-360.,-360.,-360.,+360.,+360.,+360.);
   Int_t ierr;
   vwp->SetView(10.,80.,80.,ierr);

   if (!otubep) otubep = new TTUBE("CALO","CALO","void",350.,350.,410.);
   if (!itubep) itubep = new TTUBE("CALI","CALI","void",210.,210.,270.);
   if (!orotp)  orotp  = new TRotMatrix("orot","orot",10.,80.,10.,80.,10.,80.);
   if (!onodep) {
      onodep = new TNode("ONODE","ONODE","CALO",0.,0.,0.,"orot");
      onodep->cd();
      inodep = new TNode("INODE","INODE","CALI");
   }
   onodep->Draw("pad same");

   //--
   // Daraw PFOs
   //--
   TObjArray *pfosp = cpmbp->GetOutputBufPtr();
   Int_t color  = 0;
   if (pfosp && pfosp->GetEntries()) {
      TIter next(pfosp);
      S4PFObject *pfp = 0;
      while ((pfp = static_cast<S4PFObject *>(next()))) {
         color++;
         color %= 10;
         pfp->Draw(color,opt);
      }
   }
   //--
   // Prompt
   //--
   cout << "Next? [yes/no/edit/quit] " << flush;
   const Int_t kMaxLen = 1024;
   Char_t temp[kMaxLen];
   cin.getline(temp,kMaxLen);
   TString opts(temp);
   opts.ToLower();
   if (!opts.Length()) {
      continue;
   } else if (opts[0] == 'n' || opts[0] == 'q') {
      exit(0);
   } else if (opts[0] == 'e') {
      cout << "Select \"Quit ROOT\" from \"File\" to display next" << endl;
      cout << "\"CNTRL+C\" to really quit" << endl;
      gROOT->GetApplication()->SetReturnFromRun(kTRUE);
      gSystem->Run();
      gROOT->GetApplication()->SetReturnFromRun(kFALSE);
   }
}
