#ifndef J4CALPREHIT_H
#define J4CALPREHIT_H
// $Id: J4CALPreHit.hh,v 1.5 2004/11/25 05:22:22 yamaguci Exp $
//*************************************************************************
//* --------------------
//* J4CALPreHit
//* --------------------
//* (Description)
//*     Hit class for J4CAL 	
//*	Mother class : G4VHit
//*     
//* (Update Record)
//*     2004/08/30  Ono Hiroaki J4CALPreHit class for CAL front infomation
//*     2004/11/01  Allister Sanchez  Remove redundant data/functions which are in J4VHit.
//*************************************************************************

#include "J4VHit.hh"
#include "J4Output.hh"
#include "G4THitsCollection.hh"
#include "G4HCofThisEvent.hh"
#include "G4ParticleDefinition.hh"

//=========================================================================
// TypeDef

class J4VComponent;
class J4CALPreHit;
typedef G4Allocator<J4CALPreHit> J4CALPreHitAllocator;
typedef G4THitsCollection<J4CALPreHit> J4CALPreHitBuf;

//=========================================================================
//---------------------
// class definition
//---------------------

class J4CALPreHit : public J4VHit {
public:
  J4CALPreHit();
  J4CALPreHit(J4VComponent        * comp,
              G4int                 hitID,
              const G4ThreeVector & pre,
	      const G4ThreeVector & momentum,
	      G4double              edep,
	      G4double              tof,
	      G4int                 trackID,
              G4ParticleDefinition* particle,
	      G4int                 motherTrackID );

  virtual ~J4CALPreHit();

  inline void *operator new    (size_t);
  inline void  operator delete (void* aHit);

  virtual void Output(G4HCofThisEvent* HCTE);
  virtual void Draw ();
  virtual void Print();

  // getters
  inline G4int         GetPreHitID      ()   const { return fPreHitID;        }
  inline G4ThreeVector GetInjectionPoint()   const { return GetPrePosition(); }
  
  // setters
  inline void SetPreHitID      (G4int         n)   { fPreHitID       = n;  }
  inline void SetInjectionPoint(G4ThreeVector p)   { SetPrePosition(p);    }

  static void SetOutput( J4Output *output )        { fgPreOutput = output; } 

private:
  static J4CALPreHitAllocator  fgPreHitAllocator;
  static J4Output             *fgPreOutput;       // Pointer to Output Module
  G4int                        fPreHitID;         // Pre-hit ID
};

//=====================================================================
//----------------------------------------
// inline function for J4CALPreHit
//----------------------------------------

void* J4CALPreHit::operator new(size_t)
{
  void* aHit;
  aHit = (void*)fgPreHitAllocator.MallocSingle();
  return aHit;
}

void J4CALPreHit::operator delete(void* aHit)
{
  fgPreHitAllocator.FreeSingle( (J4CALPreHit*)aHit );
}
#endif
